package fxlib.dsp;

public class Biquad
{
    private double[] x;
    private double[] y;
    private double[] b;
    private double[] a;

    public Biquad()
    {
        reset();
    }
    public void reset()
    {
        x = new double[3];
        y = new double[3];
        b = new double[3];
        a = new double[3];
    }
    public void clear()
    {
        x[0] = x[1] = x[2] = 0.0;
        y[0] = y[1] = y[2] = 0.0;
    }
    public double directFormI(double sample, boolean newSample)
    {
        if(newSample == true)
        {
            x[2] = x[1];
            x[1] = x[0];
            x[0] = sample;
            y[2] = y[1];
            y[1] = y[0];
        }
        else
        {
            x[0] = sample;
        }

        y[0] = x[0] * b[0] + x[1] * b[1] + x[2] * b[2] + y[1] * a[1] + y[2] * a[2];

        return y[0];
    }
    public void setLowpass(double period, double Q)
    {
        double w0 = (2.0 * Math.PI) / period;
        double cw0 = Math.cos(w0);
        double alpha = Math.sin(w0) / (Q + Q);
        double t = 1.0 - cw0;

        b[0] = t * 0.5;
        b[1] = t;
        b[2] = b[0];
        a[0] = 1.0 / (1.0 + alpha);
        a[1] = -(cw0 + cw0);
        a[2] = 1.0 - alpha;

        b[0] *= a[0];
        b[1] *= a[0];
        b[2] *= a[0];
        a[1] *= -a[0];
        a[2] *= -a[0];
        a[0] = 1.0;
    }
    public void setHighpass(double period, double Q)
    {
        double w0 = (2.0 * Math.PI) / period;
        double cw0 = Math.cos(w0);
        double alpha = Math.sin(w0) / (Q + Q);
        double t = 1.0 + cw0;

        b[0] = t * 0.5;
        b[1] = -t;
        b[2] = b[0];
        a[0] = 1.0 / (1.0 + alpha);
        a[1] = -(cw0 + cw0);
        a[2] = 1.0 - alpha;

        b[0] *= a[0];
        b[1] *= a[0];
        b[2] *= a[0];
        a[1] *= -a[0];
        a[2] *= -a[0];
        a[0] = 1.0;
    }
    public void setBandpass(double period, double bandwidth)
    {
        double w0 = (2.0 * Math.PI) / period;
        double cw0 = Math.cos(w0);
        double sw0 = Math.sin(w0);
        double alpha = sw0 * Math.sinh( Math.log(2.0)*0.5 * bandwidth * w0/sw0);

        b[0] = alpha;
        b[1] = 0.0;
        b[2] = -alpha;
        a[0] = 1.0 / (1.0 + alpha);
        a[1] = -(cw0 + cw0);
        a[2] = 1.0 - alpha;

        b[0] *= a[0];
        b[2] *= a[0];
        a[1] *= -a[0];
        a[2] *= -a[0];
        a[0] = 1.0;
    }
    public double getOutput()
    {
        return y[0];
    }
}
